## BEFORE YOU BEGIN
install.packages(c('gstat','sf','stars','vegan','SpatialPack','ggplot2','terra','fields'))

# Note for Linux users. If the installation of does not work, enter this command in the terminal:
# > sudo apt-get install libgdal-dev gdal-bin proj-bin libproj-dev

## PART I - LOAD NECESSARY PACKAGES

#Load each package
library(gstat)
library(stars)
library(terra)
library(sf)
library(SpatialPack)
library(fields)
library(ggplot2)


# Select the file 'Arboretum_Data.csv' on your computer. 
Arbo<-read.csv(file.choose())

# Variables names/acronyms
# Coordinates("X","Y"), "ID", Total carbon 0-15 cm of soil("tC_015"),Total carbon 15-30 cm of soil ("tC_1530"), Total carbon in forest floow ("FFtC"), Total soil carbon("tC"), Sand %("Sand"), Silt %("Silt"), Clay %("Clay"), Soil pH("pH"), Potassium ("K"), Phosphorus("P"), Magnesium ("Mg"), Calcium ("Ca"), N mineralization("Min"), Basal areas for: Red maple("RM"), Sugar maple("SuM"), Silver maple("SiM"), American beech("BE"), Red oak("RO"), White birch("WB"), Gray birch("GB"), Yellow birch("YB"), Basswood("BW"), Bitternut hickory ("BH"), Shagbark Hickory ("SH"), American elm("AE"), White ash("WA"), Black ash("BA"), Ironwood("IW"), Musclewood("MW"), Hemlock("HE") 

# Select the Arbo_mask.tif file on your computer
Arbo_mask <- read_stars(file.choose())

Arbo <- st_as_sf(Arbo, coords = c("X","Y"),crs=st_crs(Arbo_mask))

# Specify the theme to be used for plots and maps
# You can use one of the existing palettes
pal = hcl.colors # or terrain.colors, heat.colors, topo.colors, cm.colors
# Or make a custom palette
pal <- colorRampPalette(c("black","brown","orange","yellow","white"))
# Feel free to make your own custom palette

# Visualize some variables 
plot(Arbo['Ca'],pal=pal,pch=19)
plot(Arbo['tC'],pal=pal,pch=19)
plot(Arbo['pH'],pal=pal,pch=19)


##============================================================================
## PART 2 - EXPERIMENTAL VARIOGRAM AND AUTOCORRELATION MEASURES
##============================================================================

# Semi-variogram for % Sand. 
# Variogram cloud
vario.SandCloud <- variogram(Sand~1,Arbo,cloud=TRUE, cutoff=350)
# We choose a cutoff of roughly half of the square root of the study area's surface
plot(vario.SandCloud)

# Average at each lag ('classical' variogram)
vario.Sand <- variogram(Sand~1,Arbo, cutoff=350, width=50)
# We choose a width that will give us enough pairs of points per distance class 
# and that gives a variogram with roughly 5-15 points
# try changing the cutoff and the width to see the effect
plot(vario.Sand)

# Variogram for total carbon
vario.tC <- variogram(tC ~ 1, Arbo, cutoff=350, width=50)
plot(vario.tC)

# Choose the file GearyC.R
source(file.choose())
# Choose the file MoranI.R
source(file.choose())

# Geary's C
corgramC<-GearyC(Arbo,'Sand',seq(0,350,50))
# Error bars that don't touch the horizontal line=1 are considered significant
# There is spatial autocorrelation up to the first non-significant distance
plot(corgramC)

# Moran's I
corgramI<-MoranI(Arbo,'Sand',seq(0,350,50))
# Error bars that don't touch the horizontal line=0 are considered significant
# There is spatial autocorrelation up to the first non-significant distance
plot(corgramI)


##====================================================
## PART 3 - UNCONDITIONAL SIMULATIONS
##====================================================

# Unconditional simulations
# Pure nugget (no spatial structure)
v <- vgm(1,'Nug',0)
g.dummy <- gstat(formula = z~1, locations=Arbo, dummy = TRUE, beta = 0,model = v, nmax = 20)
g.prdNug<-predict(g.dummy,Arbo_mask,nsim=9)
plot(g.prdNug,col=pal(20))

# Spherical model with 200 m range. 
v <- vgm(1, "Sph", 200)
g.dummy <- gstat(formula = z~1,dummy = TRUE, beta = 0,model = v, nmax = 20)
g.prdSph<-predict(g.dummy,Arbo_mask,nsim=9)
plot(g.prdSph,col=pal(20))

# Gaussian model with 200 m range. 
v <- vgm(1, "Gau", 200, nug = 0.0001)
# Note: we add a small nugget effect here for mathematical reasons (the variance-covariance matrix would not be invertible due to covariances at small distances being too similar).
g.dummy <- gstat(formula = z~1, dummy = TRUE, beta = 0,model = v, nmax = 20)
g.prdGau<-predict(g.dummy,Arbo_mask,nsim=9)
plot(g.prdGau,col=pal(20))

# VARIOGRAM MODELING
vario.Sand <- variogram(Sand~1, Arbo, cutoff=350, width=50)
plot(vario.Sand)
v <- vgm(500, "Sph", 200, nug = 250)
#The values selected here are first rough approximations, they don't have to be 
#accurate. Try changing them to see if there will be an effect
model.Sand = fit.variogram(vario.Sand, model = v)
#Values are adjusted by the model at this step
plot(vario.Sand,model=model.Sand)
# To give an idea of the 'goodness of fit': 
attr(model.Sand,'SSErr') # Sums of squared differences between the variogram model and the experimental variogram. 

# Variogram for Sugar maple
vario.SuM <- variogram(SuM ~ 1, Arbo, cutoff=350, width=50)
plot(vario.SuM)
v <- vgm(500, "Sph", 200, nug = 250) # 'Initial' values can be guessed by visually inspecting the experimental variogram
model.SuM = fit.variogram(vario.SuM, model = v) # Look at model.SuM to see estimated values. 
plot(vario.SuM, model=model.SuM)

# Total carbon
vario.tC <- variogram(tC ~ 1, Arbo, cutoff=350, width=50)
plot(vario.tC)
v <- vgm(1, "Nug", 0)
# Pure nugget effect with no spatial component
model.tC = fit.variogram(vario.tC, model = v)
plot(vario.tC, model=model.tC)

# Calcium
vario.Ca <- variogram(Ca ~ 1, Arbo,cutoff=350, width=50)
plot(vario.Ca)
v <- vgm(1500000, "Sph", 200, nug = 250)
model.Ca = fit.variogram(vario.Ca, model = v)
plot(vario.Ca, model=model.Ca)

##===========================================================
## PART 4 - INTERPOLATION - KRIGING - CONDITIONAL SIMULATIONS
##===========================================================

# Inverse Distance Weighting
idwr <- idw(pH ~ 1, Arbo, Arbo_mask,idp=2)
plot(idwr['var1.pred'],col=pal(20))

# Simple Kriging
vario.pH <- variogram(pH~1,Arbo, cutoff=350, width=50)
v <- vgm(500, "Sph", 200, nug = 250)
model.pH = fit.variogram(vario.pH, model = v)
kri <- krige(pH ~ 1, Arbo, newdata=Arbo_mask, model = model.pH,beta=1)
plot(kri['var1.pred'],col=pal(20))

# Ordinary Kriging
kri <- krige(pH ~ 1, Arbo, Arbo_mask, model = model.pH,maxdist=200)
plot(kri['var1.pred'],col=pal(20)) # Map of kriging estimates
plot(kri['var1.var'],col=pal(20)) # Map of kriging variances

# Save the kriged estimates to a TIF file in your working directory
write_stars(kri['var1.pred'],'phKri.tif')

# Leave-one-out Cross-validation
# We remove one observation and then try to estimate it's value. 
crossval.pH <- krige(pH ~ 1, Arbo[-1,], Arbo[1,], model = model.pH, maxdist=200)
for (i in 2:nrow(Arbo)){
  k <- krige(pH ~ 1, Arbo[-i,], Arbo[i,], model = model.pH, maxdist=200)
  crossval.pH=rbind(crossval.pH,k);
}
#Comparison of actual vs predicted values
plot(Arbo$pH,crossval.pH$var1.pred)
cor(Arbo$pH,crossval.pH$var1.pred) #Correlation coefficient between estimated and actual values

# Five-fold Cross-validation
RMSE <- function(observed, predicted) {
  sqrt(mean((predicted - observed)^2, na.rm=TRUE))
}
kf <- sample(1:5, nrow(Arbo), replace=TRUE)
rmse <- rep(NA, 5)
for (k in 1:5) {
  test <- Arbo[kf == k, ]
  train <- Arbo[kf != k, ]
  kr <- krige(pH ~ 1, train, train, model = model.pH, maxdist=200)
  rmse[k] <- RMSE(test$pH, kr$var1.pred)
}
rmse
#This value can be compared between models to see which model performs best
mean(rmse)


# CONDITIONAL GAUSSIAN SIMULATIONS
#Calcium
vario.Ca <- variogram(Ca ~ 1, Arbo,cutoff=350, width=50)
plot(vario.Ca)
v <- vgm(500, "Sph", 200, nug = 250)
model.Ca = fit.variogram(vario.Ca, model = v)
# Here, we add nsim=9 to specify number of desired simulations
Ca.sim <- krige(formula = Ca~1, Arbo, Arbo_mask, model = model.Ca,
                nmax = 15, nsim = 9)
plot(Ca.sim,col=pal(20))

# pH
vario.pH <- variogram(pH ~ 1, Arbo,cutoff=350, width=50)
plot(vario.pH)
v <- vgm(500, "Gau", 200, nug = 250)
model.pH = fit.variogram(vario.pH, model = v)
pH.sim <- krige(formula = pH~1, Arbo, Arbo_mask, model = model.pH,
                nmax = 35, nsim = 9)
plot(pH.sim,col=pal(20))

##====================================================
## PART 5 - BIVARIATE ANALYSIS
##====================================================

rm(Arbo.g)
# Note: we use the scale() fonction here to standardize the data, which is common practice in multivariate analysis. 
Arbo.g <- gstat(id="Wa", formula=scale(WA)~1, data=Arbo, nmax = 10)
Arbo.g <- gstat(Arbo.g, "SuM", scale(SuM)~1, Arbo, nmax = 10)
Xmodel <- vgm(0.3, "Sph", 100, nug = 0.1)
Xmodel <- vgm(0.7, "Sph", 250,add.to=Xmodel)
Xvario <- variogram(Arbo.g, cutoff=300,40)
XArbo.fit = fit.lmc(Xvario, Arbo.g,Xmodel,fit.lmc=TRUE)

# Cross-variogram
plot(Xvario, XArbo.fit)
xv=Xvario$id=='Wa.SuM'
v1=Xvario$id=='Wa'
v2=Xvario$id=='SuM'

# Calculation of hulls of perfect correlations
hulls=sqrt(Xvario[v1,3]*Xvario[v2,3])
hulls=cbind(hulls,-hulls)

plot(Xvario[xv,2],Xvario[xv,3],ylim=range(hulls))
par(new=FALSE)
lines(Xvario[v1,2],hulls[,1],col='red')
lines(Xvario[v1,2],hulls[,2],col='red')

# Codispersion coefficient (values between 0 et 1 at each lag)
codcoef<-Xvario[xv,3]/sqrt(Xvario[v1,3]*Xvario[v2,3])
plot(Xvario[v1,2],codcoef)

# Correlation with Dutilleul's modified T-test
# Sugar maple and pH
corr1<-modified.ttest(Arbo$SuM,Arbo$pH, st_coordinates(Arbo))
print(corr1) # Note the really low effective sample size (adjusted degrees of freedom - DF), due to the strong spatial correlation in the variables

# Sugar maple and total soil carbon
corr2<-modified.ttest(Arbo$SuM,Arbo$tC, st_coordinates(Arbo))
print(corr2) # Weak spatial structure for Carbon = effective sample size (adjusted degrees of freedom - DF) near the actual sample size value. 

